Package: ideanet
Type: Package
Title: Integrating Data Exchange and Analysis for Networks ('ideanet')
Version: 1.1.1
Date: 2025-06-02
Description: A suite of convenient tools for social network analysis geared toward students, entry-level users, and non-expert practitioners. ‘ideanet’ features unique functions for the processing and measurement of sociocentric and egocentric network data. These functions automatically generate node- and system-level measures commonly used in the analysis of these types of networks. Outputs from these functions maximize the ability of novice users to employ network measurements in further analyses while making all users less prone to common data analytic errors. Additionally, ‘ideanet’ features an R Shiny graphic user interface that allows novices to explore network data with minimal need for coding.
Authors@R: 
                c(person(given = "Tom",
                        family = "Wolff",
                        role = c("aut", "cre"),
                        email = "tom.wolff@northwestern.edu",
                        comment = c(ORCID = "0000-0002-4884-251X")),
                 person(given = c("Jonathan", "Howard"),
                        family = "Morgan",
                        role = "aut",
                        email = "jonathan.h.morgan@gmail.com",
                        comment = c(ORCID = "0000-0001-5181-9903")),
                 person(given = "Gabriel",
                        family = "Varela",
                        role = "aut",
                        email = "gabriel.varela@duke.edu",
                        comment = c(ORCID = "0000-0003-2800-1577")),
                 person(given = "Kieran",
                        family = "Lele",
                        role = "aut"),
                 person(given = "Ethan",
                        family = "Bhojani",
                        role = "aut"),
                 person(given = "Emily",
                        family = "Heraty",
                        role = "aut",
                        email = "emily.heraty@duke.edu"),
                 person(given = "Dana",
                        family = "Pasquale",
                        role = "aut",
                        email = "dana.pasquale@duke.edu",
                        comment = c(ORCID = "0000-0001-6686-7844")),
                 person(given = "Peter",
                        family = "Mucha",
                        role = "aut",
                        email = "Peter.J.Mucha@dartmouth.edu",
                        comment = c(ORCID = "0000-0002-0648-7230")),
                 person(given = "James",
                        family = "Moody",
                        role = "aut",
                        email = "jmoody77@duke.edu",
                        comment = c(ORCID = "0000-0002-3311-4173")))
Maintainer: Tom Wolff <tom.wolff@northwestern.edu>
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
Imports: CliquePercolation, cluster, colorspace, concorR, cowplot,
        data.table, dplyr, forcats, ggplot2, ggthemes, grDevices,
        gridGraphics, igraph (>= 2.1.0), intergraph, jsonlite,
        magrittr, Matrix, methods, moments, network, readxl, reshape2,
        rlang, RSpectra, shiny, sna, stringr, tibble, tidyr, tidyselect
Suggests: DT, devtools, egor, ergm, shinythemes, shinyWidgets, knitr,
        rmarkdown, shinycssloaders, visNetwork
RoxygenNote: 7.3.2
Depends: R (>= 3.5.0), igraphdata
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-07-03 16:26:32 UTC; wms1212
Author: Tom Wolff [aut, cre] (ORCID: <https://orcid.org/0000-0002-4884-251X>),
  Jonathan Howard Morgan [aut] (ORCID:
    <https://orcid.org/0000-0001-5181-9903>),
  Gabriel Varela [aut] (ORCID: <https://orcid.org/0000-0003-2800-1577>),
  Kieran Lele [aut],
  Ethan Bhojani [aut],
  Emily Heraty [aut],
  Dana Pasquale [aut] (ORCID: <https://orcid.org/0000-0001-6686-7844>),
  Peter Mucha [aut] (ORCID: <https://orcid.org/0000-0002-0648-7230>),
  James Moody [aut] (ORCID: <https://orcid.org/0000-0002-3311-4173>)
Repository: CRAN
Date/Publication: 2025-07-03 16:50:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:43:57 UTC; unix
