# Generated by roxygen2: do not edit by hand

export("%>%")
export(CHAMP)
export(comm_detect)
export(ego_homophily)
export(ego_netwrite)
export(ego_reshape)
export(ei_index)
export(euclidean_distance)
export(get_CHAMP_map)
export(get_egograph)
export(get_partitions)
export(h_index)
export(ideanetViz)
export(iqv)
export(nc_merge)
export(nc_read)
export(netread)
export(netwrite)
export(pearson_phi)
export(qap_run)
export(qap_setup)
export(role_analysis)
import(igraphdata)
importFrom(data.table,":=")
importFrom(magrittr,"%>%")
importFrom(rlang,.data)
